<?xml version="1.0"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" >
    <xsl:output method="html"  encoding="utf-16"/>
    <xsl:template match="Report">
        <head>
			<title>Flow Analysis profinig results</title>
			
			<script type="text/javascript">
			
				function hide_row(rowId)
				{
					row = document.getElementById(rowId);
					row.style.display = "none";
				}
				
				function show_row(rowId)
				{
					row = document.getElementById(rowId);
					row.style.display = "";
				}
				
				function has_children(rowId)
				{
					row = document.getElementById(rowId + '_0');
					return (row != null);
				}
				
				function get_children(rowId)
				{
					var result = new Array();
					var childrensArray = new Array();
					var i = 0;
					while(true)
					{
						childRowId = rowId + '_' + i;
						childRow = document.getElementById(childRowId)
						if(childRow)
						{
							result[i] = childRowId;
							childrensArray = childrensArray.concat(get_children(childRowId));
						}
						else
						{
							break;
						}
						i++
					}
					result = result.concat(childrensArray);
					return (result);
				}
				
				function toggle_row(rowId)
				{
					var hasChildren = has_children(rowId);
					if(!hasChildren)
					{
						return;
					}
					
					children = get_children(rowId);
					var childrenElement = document.getElementById(children[0]);
					var hidden = (childrenElement.style.display == "none");
					
					for(i = 0; i != children.length; i++)
					{
						if(!hidden)
						{
							hide_row(children[i]);
						}
						else
						{
							show_row(children[i]);
						}
					}
					
				}
			</script>
			
			
			<style type="text/css">
				BODY {font-family: Arial, Helvetica, sans-serif;font-size: 11px;font-style: normal;color: #000000;}
				table.none { margin: 0em; }
				td.none, th.none { padding: .0em; border: 0px; } 
				
				table { margin: 1em; border-collapse: collapse; }
				td, th { padding: .3em; border: 1px #ccc solid; } 
				thead { background: #999999; } 
				#time{ padding: .3em; border: 1px #ccc solid; text-align: center;} 
				.pointer {cursor: pointer; font-weight: bold;}
			</style>
        </head>
        <body>
		<table class="none" width="95%" cellpadding="0" cellspacing="0">
			<tr>
				<td class="none" background="rep_header_expander.jpg" width="50%" align="left"><img src="rep_header.jpg" alt="rep_header.jpg" /></td>
				<td class="none" background="rep_header_expander.jpg" width="50%" align="right"><img src="rep_header_logo.jpg" alt="rep_header_logo.jpg" /></td>
			</tr>
		</table>
		
		<table class="none" width="95%" border="1" bordercolor="#93a9b9" cellpadding="0" cellspacing="0">
			<tr>
				<td class="none" width="16%" align="center"><font class="time"><xsl:value-of select="@date"/></font></td>
				<td class="none" align="center" bgcolor="#b7cfe1">
					<table class="none" width="100%" border="0" bordercolor="#93a9b9" cellpadding="0" cellspacing="0">
						<tr>
							<td class="none" width="81%" align="center" bgcolor="#b7cfe1"><b>Profiling Results</b></td>
							<td class="none" bgcolor="#b7cfe1"></td>
						</tr>
					</table>
				</td>
			</tr>
		</table>
		
		<xsl:apply-templates />
		
		<table class="none" width="95%" border="1" bordercolor="#93a9b9" bgcolor="#b7cfe1" >
			<tr>
				<td class="none" width="90%" bgcolor="#b7cfe1"><a name="explanation"><p style="margin: 5px;">
					<b>*</b>Values in 'Time' column are milliseconds of CPU core execution time. 
					The lower bound of the actual time can be estimated by dividing the 'Time' value by 
					number of cores used for the analysis (by default number of cores of CPU).
					<br/><br/>This report can be used to estimate proportions of time spent in analysis
					of certain rules or files, but it is not guaranteed that excluding a rule(s) or file(s) 
					from the analysis will decrease the time by exactly the corresponding value(s) from the 'Time' 
					column. This is due to number of actions that have to be performed during analysis even when a 
					rule or file is excluded. 
				</p></a></td>
			</tr>
		</table>
		
		<table class="none" width="95%" cellpadding="0" cellspacing="0">
			<tr>
				<td class="none" background="rep_footer_expander.jpg" width="100%" align="left"><img src="rep_footer.jpg" alt="rep_footer.jpg" /></td>
			</tr>
		</table>
        </body>
    </xsl:template>
	
	
	<xsl:template match="Category">
	<xsl:variable name="tableId" select="count(preceding-sibling::Category)" />
        <table id="{count(preceding-sibling::Category)}">
			<thead>
				<tr>
					<th><xsl:value-of select="@name"/></th>
					<th>Time [ms] <a href="#explanation">*</a></th>
				</tr>
			</thead>
			<xsl:for-each select="Item">
			    <xsl:sort select="@time" data-type="number" order="descending" />
				<xsl:call-template name="itemTemplate">
					<xsl:with-param name="whiteSpace" select="''" />
					<xsl:with-param name="parentId" select="$tableId"/>
				</xsl:call-template>
			</xsl:for-each>
        </table>
	</xsl:template>
    <xsl:template name="itemTemplate">
		<xsl:param name="whiteSpace" />
		<xsl:param name="parentId" />
		
		<xsl:variable name="itemId" select="concat($parentId,'_',count(preceding-sibling::Item))" />
		<tr id="{$itemId}">
			<xsl:if test="count(child::Item) != 0">
				<xsl:attribute name="class">pointer</xsl:attribute>
			</xsl:if>
			<td onclick="toggle_row('{$itemId}')"><xsl:value-of select="$whiteSpace"/> <xsl:value-of select="@name"/></td>
            <td id="time"><xsl:value-of select="@time"/></td>
		</tr>
		<xsl:for-each select="Item">
		    <xsl:sort select="@time" data-type="number" order="descending" />
            <xsl:call-template name="itemTemplate">
			    <xsl:with-param name="whiteSpace" select="concat('-',$whiteSpace)" />
				<xsl:with-param name="parentId" select="$itemId"/>
			</xsl:call-template>
		</xsl:for-each>
    </xsl:template>
</xsl:stylesheet>
