<?xml version="1.0"?>
<!-- in case of any change in xsl/xml file modify all connected xsl and xml files from this package  -->

<xsl:stylesheet 
  version="1.0" 
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:fo="http://www.w3.org/1999/XSL/Format"
  xmlns:saxon="http://icl.com/saxon"
  xmlns:fox="http://xml.apache.org/fop/extensions"
  extension-element-prefixes="saxon">
  
  <xsl:param name="output_dir"></xsl:param>
  <xsl:param name="fontfamily"></xsl:param>
 
  <xsl:template match="html">    

    <fo:root xmlns:fo="http://www.w3.org/1999/XSL/Format"
      xmlns:fox="http://xml.apache.org/fop/extensions">
      <xsl:if test="$fontfamily">
	      <xsl:attribute name="font-family">          
	        <xsl:value-of select="$fontfamily"/>          
	      </xsl:attribute>
	  </xsl:if>              
      <fo:layout-master-set>        
        
            <fo:simple-page-master master-name="first"
              page-height="29.7cm" page-width="21cm">
              <fo:region-body margin-top="1.5cm" margin-bottom="1.5cm">
                <xsl:call-template name="bodycolor"/>
              </fo:region-body>
              <fo:region-before region-name="rb-right" 
                extent="1.5cm">
                <xsl:call-template name="bodycolor"/>
              </fo:region-before>
              <fo:region-after region-name="ra-right" 
                extent="1.5cm">
                <xsl:call-template name="bodycolor"/>
              </fo:region-after>
            </fo:simple-page-master>

            <fo:simple-page-master master-name="left"
              page-height="29.7cm" page-width="21cm">

              <fo:region-body margin-top="1.5cm" margin-bottom="1.5cm">
                <xsl:call-template name="bodycolor"/>
                </fo:region-body>
              <fo:region-before region-name="rb-left" 
                extent="1.5cm">
                <xsl:call-template name="bodycolor"/>
              </fo:region-before>
              <fo:region-after region-name="ra-left" 
                extent="1.5cm">
                <xsl:call-template name="bodycolor"/>
              </fo:region-after>
            </fo:simple-page-master>

            <fo:simple-page-master master-name="right"
              page-height="29.7cm" page-width="21cm">

              <fo:region-body margin-top="1.5cm" margin-bottom="1.5cm">
                <xsl:call-template name="bodycolor"/>
              </fo:region-body>
              <fo:region-before region-name="rb-right" 
                extent="1.5cm">
                <xsl:call-template name="bodycolor"/>
              </fo:region-before>
              <fo:region-after region-name="ra-right" 
                extent="1.5cm">
                <xsl:call-template name="bodycolor"/>
              </fo:region-after>
            </fo:simple-page-master>
        

        <fo:page-sequence-master master-name="standard">
          <fo:repeatable-page-master-alternatives>
            <fo:conditional-page-master-reference 
              master-reference="first" 
              page-position="first"/>
            <fo:conditional-page-master-reference 
              master-reference="left" 
              odd-or-even="even"/>
            <fo:conditional-page-master-reference 
              master-reference="right" 
              odd-or-even="odd"/>
          </fo:repeatable-page-master-alternatives>
        </fo:page-sequence-master>

      </fo:layout-master-set>

      <!--xsl:call-template name="generate-bookmarks"/ -->

      <fo:page-sequence master-reference="standard">

        <!-- ============================================
    We define the static content for the headers 
    and footers of the various page layouts first. 
    =============================================== -->

        <fo:static-content flow-name="ra-right">
          <fo:block font-size="8pt" text-align-last="end">	  
            <fo:table table-layout="fixed">
              <fo:table-column column-width="466pt"/>
              <fo:table-column column-width="72pt"/>
              <fo:table-body>
                <fo:table-row>
                  <fo:table-cell>
                    <fo:block text-align="start" padding-before="8pt">
                      <xsl:value-of select="/html/head/title"/>
                    </fo:block>
                  </fo:table-cell>
                  <fo:table-cell>
                    <fo:block text-align="end" padding-before="8pt">
                      Page
                      <fo:page-number/>
                    </fo:block>
                  </fo:table-cell>
                </fo:table-row>
              </fo:table-body>
            </fo:table>
          </fo:block>
        </fo:static-content>
        <fo:static-content flow-name="ra-left">
          <fo:block font-size="8pt" text-align-last="end">
           <fo:table table-layout="fixed">
              <fo:table-column column-width="72pt"/>
              <fo:table-column column-width="466pt"/>
              <fo:table-body>
                <fo:table-row>
                  <fo:table-cell>
                    <fo:block text-align="start" padding-before="8pt">                    
                      Page
                      <fo:page-number/>                      
                    </fo:block>
                  </fo:table-cell>
                  <fo:table-cell>
                    <fo:block text-align="end" padding-before="8pt">
                      <xsl:value-of select="/html/head/title"/>
                    </fo:block>
                  </fo:table-cell>
                </fo:table-row>
              </fo:table-body>
            </fo:table>
          </fo:block>
        </fo:static-content>

        <xsl:apply-templates select="body"/>
      </fo:page-sequence>
    </fo:root>
  </xsl:template>

  <!-- ============================================
    Templates for individual HTML elements begin
    here. 
    =============================================== -->

  <!-- ============================================
    Processing for the anchor tag is complex.  First
    of all, if this is a named anchor, we write an empty
    <fo:block> with the appropriate id.  (In the special
    case that the next element is an <h1>, we ignore
    the element altogether and put the id on the <h1>.)
    Next, if this is a regular anchor and the href
    starts with a hash mark (#), we create a link with
    an internal-destination.  Otherwise, we create a
    link with an external destination. 
    =============================================== -->

  <xsl:template match="a">
    <xsl:choose>
      <xsl:when test="@name">
        <xsl:if test="not(name(following-sibling::*[1]) = 'h1')">
          <fo:block line-height="0pt" space-after="0pt" 
            font-size="0pt" id="{@name}"/>
        </xsl:if>
      </xsl:when>
      <!--xsl:when test="@href">
        <fo:basic-link color="blue">
          <xsl:choose>
            <xsl:when test="starts-with(@href, '#')">
              <xsl:attribute name="internal-destination">
                <xsl:value-of select="substring(@href, 2)"/>
              </xsl:attribute>
            </xsl:when>
            <xsl:otherwise>
              <xsl:attribute name="external-destination">
                <xsl:value-of select="@href"/>
              </xsl:attribute>
            </xsl:otherwise>
          </xsl:choose>
          <xsl:apply-templates select="*|text()"/>
        </fo:basic-link>
        <xsl:if test="starts-with(@href, '#')">          
          <fo:page-number-citation ref-id="{substring(@href, 2)}"/>
        </xsl:if>
      </xsl:when-->
    </xsl:choose>
    <xsl:apply-templates select="*|text()"/>
  </xsl:template>

  <!-- ============================================
      We render an address element in italics.
    =============================================== -->

  <xsl:template match="address">
    <fo:block font-style="italic" space-after="12pt">
      <xsl:apply-templates select="*|text()"/>
    </fo:block>
  </xsl:template>

  <!-- ============================================
    For bold elements, we just change the font-weight.
    =============================================== -->

  <xsl:template match="b">
    <fo:inline font-weight="bold">
      <xsl:apply-templates select="*|text()"/>
    </fo:inline>
  </xsl:template>

  <!-- ============================================
    The big element is handled with a relative 
    font size.  That means a <big> element inside
    another <big> element will be even bigger, just
    as it is in HTML. 
    =============================================== -->

  <xsl:template match="big">
    <fo:inline font-size="50%">
      <xsl:apply-templates select="*|text()"/>
    </fo:inline>
  </xsl:template>

  <!-- ============================================
    A blockquote is indented on both sides.
    =============================================== -->

  <xsl:template match="blockquote">
    <fo:block start-indent="1.5cm" end-indent="1.5cm"
      space-after="12pt">
      <xsl:apply-templates select="*|text()"/>
    </fo:block>
  </xsl:template>

  <!-- ============================================
    The HTML <body> element contains everything in
    the main part of the document.  This is analogous
    to the <fo:flow flow-name="xsl-region-body">
    element, so we put the main document processing here.  
    =============================================== -->

  <xsl:template match="body">

    <!-- ============================================
    Start generating the content for the main page 
    area (xsl-region-body).
    =============================================== -->

    <fo:flow flow-name="xsl-region-body">
    <fo:block start-indent="1.5cm" end-indent="1.5cm" space-after="12pt">
	   	<xsl:if test="@bgcolor">
	      <xsl:attribute name="background-color">          
	        <xsl:value-of select="@bgcolor"/>          
	      </xsl:attribute>
	    </xsl:if>
      	<!-- xsl:apply-templates select="/html/head/title"/>  -->
      	
      <!-- ============================================
    This one line of code processes everything in 
    the body of the document.  The template that
    processes the <body> element in turn processes
    everything that's inside it.
    =============================================== -->

      <xsl:apply-templates select="*|text()"/>

      <!-- ============================================
    We put an ID at the end of the document so we 
    can do "Page x of y" numbering.
    =============================================== -->      
	</fo:block>
    </fo:flow>
  </xsl:template>

  <!-- ============================================
    We handle a break element by inserting an 
    empty <fo:block>.
    =============================================== -->

  <xsl:template match="br">
    <fo:block white-space-treatment="preserve"> </fo:block>
  </xsl:template>

  <!-- ============================================
    We're handling <center> as a block element; if
    you use it, it creates a new paragraph that's 
    centered on the page. 
    =============================================== -->

  <xsl:template match="center">
    <fo:block text-align="center">
      <xsl:apply-templates select="*|text()"/>
    </fo:block>
  </xsl:template>

  <!-- ============================================
    The <cite> element is rendered in italics, 
    unless it's inside an italic (<i>) element. 
    We use the parent axis to check this. 
    =============================================== -->

  <xsl:template match="cite">
    <xsl:choose>
      <xsl:when test="parent::i">
        <fo:inline font-style="normal">
          <xsl:apply-templates select="*|text()"/>
        </fo:inline>
      </xsl:when>
      <xsl:otherwise>
        <fo:inline font-style="italic">
          <xsl:apply-templates select="*|text()"/>
        </fo:inline>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <!-- ============================================
    We render <code> inline in a monospaced font.
    =============================================== -->

  <xsl:template match="code">
    <fo:inline>
      <xsl:apply-templates select="*|text()"/>
    </fo:inline>
  </xsl:template>

  <!-- ============================================
    We don't do anything with the <dl> element, we
    just handle the elements it contains.  Notice
    that we're ignoring any text that appears 
    in the <dl> itself; I'm not sure if that's
    the right call.
    =============================================== -->

  <xsl:template match="dl">
    <xsl:apply-templates select="*"/>
  </xsl:template>

  <!-- ============================================
    A definition term is rendered in bold.  We 
    specify keep-with-next here, although it doesn't
    always work with FOP.
    =============================================== -->

  <xsl:template match="dt">
    <fo:block font-weight="bold" space-after="2pt"
      keep-with-next="always">
      <xsl:apply-templates select="*|text()"/>
    </fo:block>
  </xsl:template>

  <!-- ============================================
    We handle each <dd> element as an indented block
    beneath the defined term.  If the following 
    element is another <dd>, that means it's another
    definition for the same term.  In that case, 
    we don't put as much vertical space after the 
    block. 
    =============================================== -->

  <xsl:template match="dd">
    <fo:block start-indent="1cm">
      <xsl:attribute name="space-after">
        <xsl:choose>
          <xsl:when test="name(following::*[1]) = 'dd'">
            <xsl:text>3pt</xsl:text>
          </xsl:when>
          <xsl:otherwise>
            <xsl:text>12pt</xsl:text>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:attribute>
      <xsl:apply-templates select="*|text()"/>
    </fo:block>
  </xsl:template>

  <!-- ============================================
    The HTML <em> element is typically rendered in 
    italics. 
    =============================================== -->

  <xsl:template match="em">
    <fo:inline font-style="italic">
      <xsl:apply-templates select="*|text()"/>
    </fo:inline>
  </xsl:template>

  <!-- ============================================
    For the <font> element, we handle the color, 
    face, and size attributes.  Color will work if
    it's one of the twelve colors supported by XSL-FO
    or it's a hex value like x0033ff.  (In other words,
    if you tell FOP to set the color to PapayaWhip, 
    you're out of luck.)  The face attribute will 
    work if FOP supports it.  (There are ways to add 
    fonts to FOP, see the FOP documentation for more 
    info.)  Size is supported for values like 
    size="14pt", size="3", and size="+3".
    =============================================== -->

  <xsl:template match="font | div">  	
    <xsl:variable name="color" saxon:assignable="yes">
      <xsl:choose>
        <xsl:when test="@color">
          <xsl:value-of select="@color"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:text>black</xsl:text>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <xsl:variable name="face" saxon:assignable="yes">
      <xsl:choose>
        <xsl:when test="@face">
          <xsl:value-of select="@face"/>
        </xsl:when>
        <xsl:otherwise>        
          <xsl:value-of select="$fontfamily"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>        
    <xsl:variable name="size" saxon:assignable="yes">
      <xsl:choose>
        <xsl:when test="@size">
          <xsl:choose>
            <xsl:when test="contains(@size, 'pt')">
              <xsl:text>@size</xsl:text>
            </xsl:when>
            <xsl:when test="@size = '+1'">
              <xsl:text>50%</xsl:text>
            </xsl:when>
            <xsl:when test="@size = '+2'">
              <xsl:text>55%</xsl:text>
            </xsl:when>
            <xsl:when test="@size = '+3'">
              <xsl:text>60%</xsl:text>
            </xsl:when>
            <xsl:when test="@size = '+4'">
              <xsl:text>65%</xsl:text>
            </xsl:when>
            <xsl:when test="@size = '+5'">
              <xsl:text>70%</xsl:text>
            </xsl:when>
            <xsl:when test="@size = '+6'">
              <xsl:text>75%</xsl:text>
            </xsl:when>
            <xsl:when test="@size = '+7'">
              <xsl:text>80%</xsl:text>
            </xsl:when>
            <xsl:when test="@size = '-1'">
              <xsl:text>40%</xsl:text>
            </xsl:when>
            <xsl:when test="@size = '-2'">
              <xsl:text>35%</xsl:text>
            </xsl:when>
            <xsl:when test="@size = '-3'">
              <xsl:text>30%</xsl:text>
            </xsl:when>
            <xsl:when test="@size = '-4'">
              <xsl:text>25%</xsl:text>
            </xsl:when>
            <xsl:when test="@size = '-5'">
              <xsl:text>20%</xsl:text>
            </xsl:when>
            <xsl:when test="@size = '-6'">
              <xsl:text>15%</xsl:text>
            </xsl:when>
            <xsl:when test="@size = '-7'">
              <xsl:text>10%</xsl:text>
            </xsl:when>
            <xsl:when test="@size = '1'">
              <xsl:text>6pt</xsl:text>
            </xsl:when>
            <xsl:when test="@size = '2'">
              <xsl:text>7pt</xsl:text>
            </xsl:when>
            <xsl:when test="@size = '3'">
              <xsl:text>8pt</xsl:text>
            </xsl:when>
            <xsl:when test="@size = '4'">
              <xsl:text>10pt</xsl:text>
            </xsl:when>
            <xsl:when test="@size = '5'">
              <xsl:text>12pt</xsl:text>
            </xsl:when>
            <xsl:when test="@size = '6'">
              <xsl:text>14pt</xsl:text>
            </xsl:when>
            <xsl:when test="@size = '7'">
              <xsl:text>16pt</xsl:text>
            </xsl:when>
            <xsl:otherwise>
              <xsl:text>12pt</xsl:text>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:when>
        <xsl:otherwise>
          <xsl:text>8pt</xsl:text>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <xsl:if test="@class">
   	<xsl:variable name="class" select="@class"/>                     
    	<xsl:if test="/html/head/styles_pdf/*[@name=$class and @attr='div']/@color">
         	<saxon:assign name="color" select="/html/head/styles_pdf/*[@name=$class]/@color"/>
        </xsl:if>
    	<xsl:if test="/html/head/styles_pdf/*[@name=$class and @attr='font']/@color">
         	<saxon:assign name="color" select="/html/head/styles_pdf/*[@name=$class]/@color"/>
        </xsl:if>
        <xsl:if test="/html/head/styles_pdf/*[@name=$class and @attr='div']/@font-family">
         	<saxon:assign name="face" select="/html/head/styles_pdf/*[@name=$class]/@font-family"/>
        </xsl:if>
        <xsl:if test="/html/head/styles_pdf/*[@name=$class and @attr='font']/@font-family">
         	<saxon:assign name="face" select="/html/head/styles_pdf/*[@name=$class]/@font-family"/>
        </xsl:if>
		<xsl:if test="/html/head/styles_pdf/*[@name=$class and @attr='div']/@font-size">
         	<saxon:assign name="size" select="/html/head/styles_pdf/*[@name=$class]/@font-size"/>
        </xsl:if>
		<xsl:if test="/html/head/styles_pdf/*[@name=$class and @attr='font']/@font-size">
         	<saxon:assign name="size" select="/html/head/styles_pdf/*[@name=$class]/@font-size"/>
        </xsl:if>
  	</xsl:if>
  	<xsl:if test="$fontfamily">
  		<saxon:assign name="face" select="$fontfamily"/>
	</xsl:if>
	
    <xsl:if test="$size">
        <fo:inline font-family="{$face}" font-size="{$size}" color="{$color}">
      		<xsl:apply-templates select="*|text()"/>
    	</fo:inline>
    </xsl:if>    
    <xsl:if test="not($size)">
        <fo:inline font-family="{$face}" color="{$color}">
      		<xsl:apply-templates select="*|text()"/>
    	</fo:inline>
    </xsl:if>
  </xsl:template>

  <!-- ============================================
    We render the <h1> by putting a horizontal rule
    and a page break before it.  We also process 
    the id attribute; if the <h1> tag has one, we 
    use it.  If not, we see if the preceding element
    is a named anchor (<a name="x"/>).  If there is
    a named anchor before the <h1>, we use the name
    of the anchor point as the id.
    =============================================== -->

  <xsl:template match="h1">
    <fo:block break-before="page">
      <fo:leader leader-pattern="rule"/>
    </fo:block>
    <fo:block font-size="12pt" line-height="15pt"
      keep-with-next="always"
      space-after="22pt">
      <xsl:attribute name="id">
        <xsl:choose>
          <xsl:when test="@id">
            <xsl:value-of select="@id"/>
          </xsl:when>
          <xsl:when test="name(preceding-sibling::*[1]) = 'a' and
                          preceding-sibling::*[1][@name]">
            <xsl:value-of select="preceding-sibling::*[1]/@name"/>
          </xsl:when>
          <xsl:otherwise>
            <xsl:value-of select="generate-id()"/>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:attribute>
      <xsl:apply-templates select="*|text()"/>
    </fo:block>
  </xsl:template>

  <!-- ============================================
    <h2> is in a slightly smaller font than an <h1>,
    and it doesn't have a page break or a line.
    =============================================== -->

  <xsl:template match="h2">
    <fo:block font-size="10pt" line-height="14pt"
      keep-with-next="always" space-after="18pt"
      >
      <xsl:attribute name="id">
        <xsl:choose>
          <xsl:when test="@id">
            <xsl:value-of select="@id"/>
          </xsl:when>
          <xsl:otherwise>
            <xsl:value-of select="generate-id()"/>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:attribute>
      <xsl:apply-templates select="*|text()"/>
    </fo:block>
  </xsl:template>

  <!-- ============================================
    <h3> is slightly smaller than <h2>.
    =============================================== -->

  <xsl:template match="h3">
    <fo:block font-size="8pt" line-height="10pt"
      keep-with-next="always" space-after="14pt"
      >
      <xsl:attribute name="id">
        <xsl:choose>
          <xsl:when test="@id">
            <xsl:value-of select="@id"/>
          </xsl:when>
          <xsl:otherwise>
            <xsl:value-of select="generate-id()"/>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:attribute>
      <xsl:apply-templates select="*|text()"/>
    </fo:block>
  </xsl:template>

  <!-- ============================================
    <h4> is smaller than <h3>.  For the bookmarks
    and table of contents, <h4> is the lowest level
    we include.
    =============================================== -->

  <xsl:template match="h4">
    <fo:block font-size="7pt" line-height="10pt"
      keep-with-next="always" space-after="12pt"
      >
      <xsl:attribute name="id">
        <xsl:choose>
          <xsl:when test="@id">
            <xsl:value-of select="@id"/>
          </xsl:when>
          <xsl:otherwise>
            <xsl:value-of select="generate-id()"/>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:attribute>
      <xsl:apply-templates select="*|text()"/>
    </fo:block>
  </xsl:template>

  <!-- ============================================
    <h5> is pretty small, and is underlined to 
    help it stand out. 
    =============================================== -->

  <xsl:template match="h5">
    <fo:block font-size="6pt" line-height="8pt"
      keep-with-next="always" space-after="12pt"
      text-decoration="underline">
      <xsl:attribute name="id">
        <xsl:choose>
          <xsl:when test="@id">
            <xsl:value-of select="@id"/>
          </xsl:when>
          <xsl:otherwise>
            <xsl:value-of select="generate-id()"/>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:attribute>
      <xsl:apply-templates select="*|text()"/>
    </fo:block>
  </xsl:template>

  <!-- ============================================
    <h6> is the smallest heading of all, and is
    underlined and italicized.  
    =============================================== -->

  <xsl:template match="h6">
    <fo:block font-size="5pt" line-height="7pt"
      keep-with-next="always" space-after="12pt"
      font-style="italic"
      text-decoration="underline">
      <xsl:attribute name="id">
        <xsl:choose>
          <xsl:when test="@id">
            <xsl:value-of select="@id"/>
          </xsl:when>
          <xsl:otherwise>
            <xsl:value-of select="generate-id()"/>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:attribute>
      <xsl:apply-templates select="*|text()"/>
    </fo:block>
  </xsl:template>

  <!-- ============================================
    We render an <hr> with a leader.  Because <hr>
    is empty, we don't have to process any child
    elements. 
    =============================================== -->

  <xsl:template match="hr">
    <fo:block>
      <fo:leader leader-pattern="rule"/>
    </fo:block>
  </xsl:template>

  <!-- ============================================
    Italics.  You can't get much simpler than that.
    =============================================== -->

  <xsl:template match="i">
    <fo:inline font-style="italic">
      <xsl:apply-templates select="*|text()"/>
    </fo:inline>
  </xsl:template>

  <!-- ============================================
    For the <img> element, we use the src attribute
    as it comes from HTML.  We also check for any 
    width and height attributes.  If those attributes
    are there, we try to use them; height="300px" is
    used as-is, while height="300" is converted to 
    the value "300px".
    =============================================== -->

  <xsl:template match="img">
      <fo:block>
        <fo:external-graphic src="{concat($output_dir, @src)}">          
          <xsl:if test="@width">
            <xsl:attribute name="content-width">
              <xsl:choose>
                <xsl:when test="contains(@width, 'px')">
                  <xsl:value-of select="@width"/>
                </xsl:when>
                <xsl:otherwise>
                  <xsl:value-of select="concat(@width, 'px')"/>
                </xsl:otherwise>
              </xsl:choose>
            </xsl:attribute>
          </xsl:if>
          <xsl:if test="@height">
            <xsl:attribute name="content-height">
              <xsl:choose>
                <xsl:when test="contains(@height, 'px')">
                  <xsl:value-of select="@height"/>
                </xsl:when>
                <xsl:otherwise>
                  <xsl:value-of select="concat(@height, 'px')"/>
                </xsl:otherwise>
              </xsl:choose>
            </xsl:attribute>
          </xsl:if>
        </fo:external-graphic>
      </fo:block>
  </xsl:template>

  <!-- ============================================
    The <kbd> element is in a slightly larger 
    monospaced text.
    =============================================== -->

  <xsl:template match="kbd">
    <fo:inline font-size="50%">
      <xsl:apply-templates select="*|text()"/>
    </fo:inline>
  </xsl:template>

  <!-- ============================================
    We handle the <li> elements under the <ol> and 
    <ul> elements, so there's no <li> template here.
    =============================================== -->

  <!-- ============================================
    For the <nobr> element, we simply turn off the
    wrap-option. 
    =============================================== -->

  <xsl:template match="nobr">
    <fo:inline wrap-option="no-wrap">
      <xsl:apply-templates select="*|text()"/>
    </fo:inline>
  </xsl:template>

  <!-- ============================================
    We handle an ordered list with two complications:
    If the list appears inside another list (either 
    an <ol> or <ul>), we don't put any vertical space
    after it.  The other issue is that we indent the
    list according to how deeply nested the list is. 
    =============================================== -->

  <xsl:template match="ol">
    <xsl:choose>         
      <xsl:when test="(parent::ul or parent::ol) and (count(child::*) &gt; 0)">
      <fo:list-item>
        <fo:list-item-label end-indent="label-end()">
          <fo:block />             
        </fo:list-item-label>
        <fo:list-item-body>
          <fo:block>
            <xsl:call-template name="create-ordered-list"/>
          </fo:block>
        </fo:list-item-body>
      </fo:list-item>           
    </xsl:when>
    <xsl:otherwise>
      <xsl:call-template name="create-ordered-list"/>
    </xsl:otherwise>
    </xsl:choose>      
    
  </xsl:template>
  
  <xsl:template name="create-ordered-list">
    <xsl:if test="(count(child::*) &gt; 0)">
    <fo:list-block provisional-distance-between-starts="0.2cm"
      provisional-label-separation="0.2cm">
      <xsl:attribute name="space-after">
        <xsl:choose>
          <xsl:when test="ancestor::ul or ancestor::ol">
            <xsl:text>0pt</xsl:text>
          </xsl:when>
          <xsl:otherwise>
            <xsl:text>12pt</xsl:text>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:attribute>
      <xsl:attribute name="start-indent">
        <xsl:variable name="ancestors">
          <xsl:choose>
            <xsl:when test="count(ancestor::ol) or count(ancestor::ul)">
              <xsl:value-of select="0 + 
                (count(ancestor::ol) * 
                0.2 + 
                count(ancestor::ul)) * 
                0.2"/>
            </xsl:when>
            <xsl:otherwise>
              <xsl:text>0</xsl:text>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:variable>
        <xsl:value-of select="concat($ancestors, 'cm')"/>
      </xsl:attribute>
      <xsl:apply-templates select="*"/>
    </fo:list-block>
    </xsl:if>
  </xsl:template>

  <!-- ============================================
    When we handle items in an ordered list, we need
    to check if the list has a start attribute.  If
    it does, we change the starting number accordingly.
    Once we've figured out where to start counting,
    we check the type attribute to see what format
    the numbers should use.  
    =============================================== -->

  <xsl:template match="ol/li">
    <fo:list-item>
      <fo:list-item-label end-indent="label-end()">
        <fo:block>
          <xsl:variable name="value-attr">
            <xsl:choose>
              <xsl:when test="../@start">
                <xsl:number value="position() + ../@start - 1"/>
              </xsl:when>
              <xsl:otherwise>
                <xsl:number value="position()"/>
              </xsl:otherwise>
            </xsl:choose>
          </xsl:variable>
          <xsl:choose>
            <xsl:when test="../@type='i'">
              <xsl:number value="$value-attr" format="i. "/>
            </xsl:when>
            <xsl:when test="../@type='I'">
              <xsl:number value="$value-attr" format="I. "/>
            </xsl:when>
            <xsl:when test="../@type='a'">
              <xsl:number value="$value-attr" format="a. "/>
            </xsl:when>
            <xsl:when test="../@type='A'">
              <xsl:number value="$value-attr" format="A. "/>
            </xsl:when>
            <xsl:otherwise>
              <xsl:number value="$value-attr" format="1. "/>
            </xsl:otherwise>
          </xsl:choose>
        </fo:block>
      </fo:list-item-label>
      <fo:list-item-body start-indent="body-start()">
        <fo:block>
          <xsl:apply-templates select="*|text()"/>
        </fo:block>
      </fo:list-item-body>
    </fo:list-item>
  </xsl:template>

  <!-- ============================================
    Your basic paragraph.
    =============================================== -->

  <xsl:template match="p">
    <fo:block font-size="7pt" line-height="10pt"
      space-after="12pt">
      <xsl:apply-templates select="*|text()"/>
    </fo:block>
  </xsl:template>

  <!-- ============================================
    Preformatted text is rendered in a monospaced
    font.  We also have to set the wrap-option
    and white-space-collapse properties.  
    =============================================== -->

  <xsl:template match="pre">
    <fo:block linefeed-treatment="preserve" white-space-collapse="false" wrap-option="wrap" white-space-treatment="preserve">      
      <xsl:apply-templates select="*|text()"/>
    </fo:block>             
  </xsl:template>

  <!-- ============================================
    Sample text is rendered in a slightly larger
    monospaced font. 
    =============================================== -->

  <xsl:template match="samp">
    <fo:inline font-size="50%">
      <xsl:apply-templates select="*|text()"/>
    </fo:inline>
  </xsl:template>

  <!-- ============================================
    The <small> element is rendered with a relative
    font size.  That means putting one <small>
    element inside another creates really small 
    text.  
    =============================================== -->

  <xsl:template match="small">
    <fo:inline font-size="30%">
      <xsl:apply-templates select="*|text()"/>
    </fo:inline>
  </xsl:template>

  <!-- ============================================
    For strikethrough text, we use the text-decoration
    property.  
    =============================================== -->

  <xsl:template match="strike">
    <fo:inline text-decoration="line-through">
      <xsl:apply-templates select="*|text()"/>
    </fo:inline>
  </xsl:template>

  <!-- ============================================
    Strongly emphasized text is simply rendered
    in bold. 
    =============================================== -->

  <xsl:template match="strong">
    <fo:inline font-weight="bold">
      <xsl:apply-templates select="*|text()"/>
    </fo:inline>
  </xsl:template>

  <!-- ============================================
    For subscript text, we use the vertical-align
    property and decrease the font size.  
    =============================================== -->

  <xsl:template match="sub">
    <fo:inline vertical-align="sub" font-size="35%">
      <xsl:apply-templates select="*|text()"/>
    </fo:inline>
  </xsl:template>

  <!-- ============================================
    Superscript text changes the vertical-align
    property also, and uses a smaller font.
    =============================================== -->

  <xsl:template match="sup">
    <fo:inline vertical-align="super" font-size="35%">
      <xsl:apply-templates select="*|text()"/>
    </fo:inline>
  </xsl:template>

  <!-- ============================================
    Tables are a hassle.  The main problem we have
    is converting the cols attribute into some 
    number of <fo:table-column> elements.  We do 
    this with a named template called build-columns.
    Once we've processed the cols attribute, we 
    invoke all of the templates for the children 
    of this element. 
    =============================================== -->

  <xsl:template match="table">    
    <xsl:if test="(count(descendant::td) &gt; 0)">
      <fo:table table-layout="fixed" margin-top="2pt" margin-bottom="2pt">
        <xsl:if test="@margin">
          <xsl:attribute name="margin-top"><xsl:value-of select="@margin"/></xsl:attribute>
          <xsl:attribute name="margin-bottom"><xsl:value-of select="@margin"/></xsl:attribute>
        </xsl:if>
      	<xsl:variable name="class" select="@class"/>
      	<xsl:variable name="border_from_style" select="/html/head/styles_pdf/*[@name=$class and @attr='table']/@border" />      
        <xsl:if test="(@border and not(@border='0')) or $border_from_style">
          <xsl:attribute name="border-style">
            <xsl:text>solid</xsl:text>
          </xsl:attribute>          
          <xsl:attribute name="border-color">
            <xsl:value-of select="@bordercolor"/>          
          </xsl:attribute>          
          <xsl:attribute name="border-start-width">
            <xsl:text>1pt</xsl:text>
          </xsl:attribute>
        </xsl:if>
        <xsl:choose>
          <xsl:when test="@cols">
            <xsl:variable name="tablewidth" saxon:assignable="yes">396</xsl:variable>          
            <xsl:call-template name="build-columns">
              <xsl:with-param name="cols" 
                select="concat(@cols, ' ')"/>
              <xsl:with-param name="tablewidth" select="$tablewidth"/>
            </xsl:call-template>
          </xsl:when>
          <xsl:otherwise>
            <fo:table-column column-width="200pt"/>
          </xsl:otherwise>
        </xsl:choose>
        
        <fo:table-body>        
          <xsl:apply-templates select="*"/>
        </fo:table-body>
      </fo:table>
    </xsl:if>
  </xsl:template>

  <!-- ============================================
    For a table cell, we put everything inside a
    <fo:table-cell> element.  We set the padding
    property correctly, then we set the border 
    style.  For the border style, we look to see if
    any of the ancestor elements we care about 
    specified a solid border.  Next, we check for the 
    rowspan, colspan, and align attributes.  Notice 
    that for align, we check this element, then go
    up the ancestor chain until we find the <table>
    element or we find something that specifies the 
    alignment. 
    =============================================== -->

  <xsl:template match="td">
    <fo:table-cell 
      padding-start="1pt" padding-end="1pt"
      padding-before="1pt" padding-after="1pt"
      start-indent="0pt" end-indent="0pt">
      <xsl:if test="@colspan">
        <xsl:attribute name="number-columns-spanned">
          <xsl:value-of select="@colspan"/>
        </xsl:attribute>
      </xsl:if>
      <xsl:if test="@rowspan">
        <xsl:attribute name="number-rows-spanned">
          <xsl:value-of select="@rowspan"/>
        </xsl:attribute>
      </xsl:if>
      <xsl:variable name="class" select="@class"/>                     
      <xsl:variable name="border_from_style" select="/html/head/styles_pdf/*[@name=$class and @attr='td']/@border" />
      <xsl:if test="(@border and not(@border='0')) or $border_from_style or
      				(../../@border and not(../../@border='0') and not(../../@rules='none'))">
        <xsl:attribute name="border-style">
          <xsl:text>solid</xsl:text>
        </xsl:attribute>
        <xsl:if test="@bordercolor">
          <xsl:attribute name="border-color">
            <xsl:value-of select="@bordercolor"/>
          </xsl:attribute>        
        </xsl:if>        
        <xsl:if test="not(@bordercolor)">
          <xsl:if test="../../@bordercolor">
            <xsl:attribute name="border-color">
              <xsl:value-of select="../../@bordercolor"/>
            </xsl:attribute>
          </xsl:if>        
        </xsl:if>
        <xsl:attribute name="border-width">
          <xsl:text>1pt</xsl:text>
        </xsl:attribute>
      </xsl:if>
      <xsl:if test="@bgcolor">
      <xsl:attribute name="background-color">          
        <xsl:value-of select="@bgcolor"/>          
      </xsl:attribute>
        </xsl:if>
        <xsl:if test="@background">
      <xsl:attribute name="background-image">          
        <xsl:value-of select="concat($output_dir, @background)"/>          
      </xsl:attribute>
        </xsl:if>
      <xsl:if test="not(@bgcolor)">
        <xsl:if test="../../@bgcolor">
          <xsl:attribute name="background-color">
            <xsl:value-of select="../../@bgcolor"/>
          </xsl:attribute>        
        </xsl:if>
      </xsl:if>
      <xsl:variable name="align">
        <xsl:choose>
          <xsl:when test="@align">
            <xsl:choose>
              <xsl:when test="@align='center'">
                <xsl:text>center</xsl:text>
              </xsl:when>
              <xsl:when test="@align='right'">
                <xsl:text>end</xsl:text>
              </xsl:when>
              <xsl:when test="@align='justify'">
                <xsl:text>justify</xsl:text>
              </xsl:when>
              <xsl:otherwise>
                <xsl:text>start</xsl:text>
              </xsl:otherwise>
            </xsl:choose>
          </xsl:when>
          <xsl:when test="parent::tr[@align]">
            <xsl:choose>
              <xsl:when test="parent::tr/@align='center'">
                <xsl:text>center</xsl:text>
              </xsl:when>
              <xsl:when test="parent::tr/@align='right'">
                <xsl:text>end</xsl:text>
              </xsl:when>
              <xsl:when test="parent::tr/@align='justify'">
                <xsl:text>justify</xsl:text>
              </xsl:when>
              <xsl:otherwise>
                <xsl:text>start</xsl:text>
              </xsl:otherwise>
            </xsl:choose>
          </xsl:when>
          <xsl:when test="parent::thead">
            <xsl:text>center</xsl:text>
          </xsl:when>
          <xsl:when test="../../table[@align]">
            <xsl:choose>
              <xsl:when test="../../table/@align='center'">
                <xsl:text>center</xsl:text>
              </xsl:when>
              <xsl:when test="../../table/@align='right'">
                <xsl:text>end</xsl:text>
              </xsl:when>
              <xsl:when test="../../table/@align='justify'">
                <xsl:text>justify</xsl:text>
              </xsl:when>
              <xsl:otherwise>
                <xsl:text>start</xsl:text>
              </xsl:otherwise>
            </xsl:choose>
          </xsl:when>
          <xsl:otherwise>
            <xsl:text>start</xsl:text>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:variable>
      <fo:block font-size="7pt" text-align="{$align}">
        <xsl:apply-templates select="*|text()"/>
      </fo:block>
    </fo:table-cell>
  </xsl:template>

  <!-- ============================================
    The rarely-used <tfoot> element contains some
    number of <tr> elements; we just invoke the 
    template for <tr> here. 
    =============================================== -->

  <xsl:template match="tfoot">
    <xsl:apply-templates select="tr"/>
  </xsl:template>

  <!-- ============================================
    If there's a <th> element, we process it just 
    like a normal <td>, except the font-weight is 
    always bold and the text-align is always center. 
    =============================================== -->

  <!--<xsl:template match="th">
    <fo:table-cell
      padding-start="3pt" padding-end="3pt"
      padding-before="3pt" padding-after="3pt">
      <xsl:if test="@border='1'">
        <xsl:attribute name="border-style">
          <xsl:text>solid</xsl:text>
        </xsl:attribute>
        <xsl:if test="@bordercolor">
          <xsl:attribute name="border-color">
            <xsl:value-of select="@bordercolor"/>
          </xsl:attribute>        
        </xsl:if>        
        <xsl:attribute name="border-width">
          <xsl:text>1pt</xsl:text>
        </xsl:attribute>
      </xsl:if>
      <fo:block font-weight="bold" text-align="center">
        <xsl:apply-templates select="*|text()"/>
      </fo:block>
    </fo:table-cell>
  </xsl:template>-->

  <!-- ============================================
    Just like <tfoot>, the rarely-used <thead> element
    contains some number of table rows.  We just 
    invoke the template for <tr> here. 
    =============================================== -->

  <!--<xsl:template match="thead">
    <xsl:apply-templates select="tr"/>
  </xsl:template>-->

  <!-- ============================================
    The title of the document is rendered in a large
    bold font, centered on the page.  This is the 
    <title> element in the <head> in <html>.
    =============================================== -->

  <xsl:template match="title">
    <fo:block space-after="18pt" line-height="14pt" 
      font-size="8pt" font-weight="bold" text-align="center">
      <xsl:apply-templates select="*|text()"/>
    </fo:block>
  </xsl:template>

  <!-- ============================================
    For an HTML table row, we create an XSL-FO table
    row, then invoke the templates for everything 
    inside it. 
    =============================================== -->

  <xsl:template match="tr">
    <xsl:if test="(count(descendant::td) &gt; 0)">
      <fo:table-row>
      	<xsl:if test="@bgcolor">
	      	<xsl:attribute name="background-color">
	        <xsl:value-of select="@bgcolor"/>          
	      	</xsl:attribute>
        </xsl:if>
        <xsl:apply-templates select="*|text()"/>
      </fo:table-row>
    </xsl:if>
  </xsl:template>

  <!-- ============================================
    Teletype text is rendered in a monospaced font.
    =============================================== -->

  <xsl:template match="tt">
    <fo:inline >
      <xsl:apply-templates select="*|text()"/>
    </fo:inline>
  </xsl:template>

  <!-- ============================================
    For underlined text, we use the text-decoration
    property.
    =============================================== -->

  <xsl:template match="u">
    <fo:inline text-decoration="underline">
      <xsl:apply-templates select="*|text()"/>
    </fo:inline>
  </xsl:template>

  <!-- ============================================
    The unordered list is pretty straightforward; 
    the only complication is calculating the space-
    after and start-indent properties.  If this 
    list is inside another list, we don't put any 
    space after this one, and we calculate the 
    indentation based on the nesting level of this 
    list. 
    =============================================== -->

  <xsl:template match="ul">
    <xsl:variable name="padding" saxon:assignable="yes">0.2</xsl:variable>
    <xsl:if test="string-length(@pdfpadding) > 0">
      <saxon:assign name="padding" select="./@pdfpadding" />
    </xsl:if>
    <xsl:choose>         
      <xsl:when test="(parent::ul or parent::ol) and (count(child::*) &gt; 0)">
        <fo:list-item>
          <fo:list-item-label end-indent="label-end()">
            <fo:block />             
          </fo:list-item-label>
          <fo:list-item-body>
            <fo:block>
              <xsl:call-template name="create-unordered-list">
                <xsl:with-param name="padding"><xsl:value-of select="$padding"/></xsl:with-param>
              </xsl:call-template>
            </fo:block>
          </fo:list-item-body>
        </fo:list-item>           
      </xsl:when>
      <xsl:otherwise>
        <xsl:call-template name="create-unordered-list">
          <xsl:with-param name="padding"><xsl:value-of select="$padding"/></xsl:with-param>
        </xsl:call-template>
        
      </xsl:otherwise>
    </xsl:choose>
    
    
  </xsl:template>
  
  <xsl:template name="create-unordered-list">
  <xsl:param name="padding"/>
    <xsl:if test="(count(child::*) &gt; 0)">
    <fo:list-block provisional-distance-between-starts="0.2cm"
      provisional-label-separation="0.2cm">
      <xsl:attribute name="space-after">
        <xsl:choose>
          <xsl:when test="ancestor::ul or ancestor::ol">
            <xsl:text>0pt</xsl:text>
          </xsl:when>
          <xsl:otherwise>
            <xsl:text>12pt</xsl:text>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:attribute>
      <xsl:attribute name="start-indent">
        <xsl:variable name="ancestors">
          <xsl:choose>
            <xsl:when test="count(ancestor::ol) or count(ancestor::ul)">
              <xsl:value-of select="(count(ancestor::ol) + count(ancestor::ul)) * $padding"/>
            </xsl:when>
            <xsl:otherwise>
              <xsl:text>0</xsl:text>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:variable>
        <xsl:value-of select="concat($ancestors, 'cm')"/>
      </xsl:attribute>
      <xsl:apply-templates select="*"/>
    </fo:list-block>
    </xsl:if>
  </xsl:template>

  <!-- ============================================
    List items inside unordered lists are easy; we
    just have to use the correct Unicode character
    for the bullet.  
    =============================================== -->

  <xsl:template match="ul/li">
    <fo:list-item>
      <fo:list-item-label end-indent="label-end()">
        <fo:block><!--&#x2022;--></fo:block>
      </fo:list-item-label>
      <fo:list-item-body start-indent="body-start()">
        <fo:block>
          <xsl:apply-templates select="*|text()"/>
        </fo:block>
      </fo:list-item-body>
    </fo:list-item>
  </xsl:template>

  <!-- ============================================
    The <var> element is rendered in italics. 
    =============================================== -->

  <xsl:template match="var">
    <fo:inline font-style="italic">
      <xsl:apply-templates select="*|text()"/>
    </fo:inline>
  </xsl:template>

  <!-- ============================================
    Named templates
    =============================================== -->


  <!-- ============================================
    This complicated template creates the PDF bookmarks
    supported by FOP.  For each <h1> element, it 
    creates a bookmark for that element, then it
    looks for all of the <h2> elements beneath it.
    For each <h2>, it looks for all the <h3> 
    elements beneath it, etc.  We use generate-id()
    extensively so we can determine whether a given
    <h1> or <h2> or <h3> is one we've seen before. 
    The end result of all this hard work is a set of
    hierarchical bookmarks that are nested within each 
    other.
    =============================================== -->

  <xsl:template name="generate-bookmarks">
    <fox:outline internal-destination="TableOfContents">
      <fox:label>Table of Contents</fox:label>
    </fox:outline>
    <xsl:for-each select="/html/body//h1">
      <xsl:variable name="current-h1" select="generate-id()"/>
      <fox:outline>
        <xsl:attribute name="internal-destination">
          <xsl:choose>
            <xsl:when test="@id">
              <xsl:value-of select="@id"/>
            </xsl:when>
            <xsl:when test="name(preceding-sibling::*[1]) = 'a' and
                            preceding-sibling::*[1][@name]">
              <xsl:value-of select="preceding-sibling::*[1]/@name"/>
            </xsl:when>
            <xsl:otherwise>
              <xsl:value-of select="generate-id()"/>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:attribute>
        <fox:label>
          <xsl:value-of select="."/>
        </fox:label>
        <xsl:for-each select="following-sibling::h2">
          <xsl:variable name="current-h2" select="generate-id()"/>
          <xsl:if 
            test="generate-id(preceding-sibling::h1[1]) = $current-h1">
            <fox:outline>
              <xsl:attribute name="internal-destination">
                <xsl:choose>
                  <xsl:when test="@id">
                    <xsl:value-of select="@id"/>
                  </xsl:when>
                  <xsl:otherwise>
                    <xsl:value-of select="$current-h2"/>
                  </xsl:otherwise>
                </xsl:choose>
              </xsl:attribute>
              <fox:label>
                <xsl:value-of select="."/>
              </fox:label>
              <xsl:for-each select="following-sibling::h3">
                <xsl:variable name="current-h3" select="generate-id()"/>
                <xsl:if 
                  test="generate-id(preceding-sibling::h2[1]) = $current-h2">
                  <fox:outline>
                    <xsl:attribute name="internal-destination">
                      <xsl:choose>
                        <xsl:when test="@id">
                          <xsl:value-of select="@id"/>
                        </xsl:when>
                        <xsl:otherwise>
                          <xsl:value-of select="$current-h2"/>
                        </xsl:otherwise>
                      </xsl:choose>
                    </xsl:attribute>
                    <fox:label>
                      <xsl:value-of select="."/>
                    </fox:label>
                    <xsl:for-each select="following-sibling::h4">
                      <xsl:if 
                        test="generate-id(preceding-sibling::h3[1]) = $current-h3">
                        <fox:outline>
                          <xsl:attribute name="internal-destination">
                            <xsl:choose>
                              <xsl:when test="@id">
                                <xsl:value-of select="@id"/>
                              </xsl:when>
                              <xsl:otherwise>
                                <xsl:value-of select="$current-h3"/>
                              </xsl:otherwise>
                            </xsl:choose>
                          </xsl:attribute>
                          <fox:label>
                            <xsl:value-of select="."/>
                          </fox:label>
                        </fox:outline>
                      </xsl:if>
                    </xsl:for-each>
                  </fox:outline>
                </xsl:if>
              </xsl:for-each>
            </fox:outline>
          </xsl:if>
        </xsl:for-each>
      </fox:outline>
    </xsl:for-each>
  </xsl:template>
  
  <xsl:template match="BREAK-TABLE">
  	<BREAK-TABLE/>
  </xsl:template>

  <!-- ============================================
    This template generates an <fo:table-column>
    element for each token in the cols attribute of
    the HTML <table> tag.  The template processes
    the first token, then invokes itself with the 
    rest of the string. 
    =============================================== -->

  <xsl:template name="build-columns">
    <xsl:param name="cols"/>    
    <xsl:param name="tablewidth"/>    
    
    <xsl:if test="string-length(normalize-space($cols))">
      <xsl:variable name="next-col">
        <xsl:value-of select="substring-before($cols, ' ')"/>
      </xsl:variable>
      <xsl:variable name="remaining-cols">
        <xsl:value-of select="substring-after($cols, ' ')"/>
      </xsl:variable>
      <xsl:choose>
        <xsl:when test="contains($next-col, 'pt')">
          <fo:table-column column-width="{$next-col}"/>
        </xsl:when>
        <xsl:when test="contains($next-col, '%')">
          <fo:table-column column-width="{$next-col}"/>
          <!--xsl:variable name="next-col-number">
            <xsl:value-of select="substring-before($next-col, '%')"/>
          </xsl:variable>
          <xsl:if test="number($next-col-number) &gt; 0">
            <fo:table-column column-width="{concat((number($tablewidth) * number($next-col-number)) div 100 , 'pt')}"/>
          </xsl:if-->
        </xsl:when>        
        <!--<xsl:when test="number($next-col) &gt; 0">
          <fo:table-column column-width="{concat($next-col, 'pt')}"/>
        </xsl:when>-->
        <!--<xsl:otherwise>
          <fo:table-column column-width="50pt"/>
        </xsl:otherwise>-->
      </xsl:choose>          
      <xsl:call-template name="build-columns">
        <xsl:with-param name="cols" select="concat($remaining-cols, ' ')"/>
        <xsl:with-param name="tablewidth" select="$tablewidth"/>
      </xsl:call-template>
    </xsl:if>
  </xsl:template>

	<xsl:template name="bodycolor">
		<xsl:if test="/html/body/@bgcolor">
	      <xsl:attribute name="background-color">          
	        <xsl:value-of select="/html/body/@bgcolor"/>          
	      </xsl:attribute>
	    </xsl:if>
	</xsl:template>
	
</xsl:stylesheet>
